#include "sdlab.h"

int tcp_connect(const char *hostname, int port)
{
  int sock = socket(AF_INET, SOCK_STREAM, IPPROTO_TCP);

  struct sockaddr_in addr;
  addr.sin_family = AF_INET;

  struct hostent *host = gethostbyname(hostname);
  addr.sin_addr = *(struct in_addr *)(host->h_addr_list[0]);
  addr.sin_port = htons(port);

  int ret = connect(sock, (struct sockaddr *)&addr, sizeof addr);
  if(ret < 0){
    return -1;
  }else{
    return sock;
  }
}

long get_filesize(const char* filename)
{
  struct stat buf;
  if(stat(filename, &buf) == 0){
    return buf.st_size;
  }else{
    return -1;
  }
}

void send_file(int sock, const char* filename)
{
  FILE *fp = fopen(filename, "r");
  while(1){
    uint8_t buf[1024];
    int size = fread(buf, sizeof(uint8_t), sizeof(buf), fp);

    if(size <= 0){
      break;
    }
    write(sock, buf, size);
  }
  fclose(fp);
}

int post_file(const char *hostname, const char *filename)
{
  int sock = tcp_connect(hostname, 80);

  char header[1024];
  char pre[1024];
  char post[1024];
  char boundary[] = "software-defined-laboratory";

  sprintf(pre, "--%s\r\n", boundary);
  sprintf(pre + strlen(pre), "Content-Disposition: form-data; ");
  sprintf(pre + strlen(pre), "name=\"userfile\"; ");
  sprintf(pre + strlen(pre), "filename=\"%s\"\r\n", filename);
  sprintf(pre + strlen(pre), "Content-Type: application/octet-stream\r\n");
  sprintf(pre + strlen(pre), "Content-Transfer-Encoding: binary\r\n\r\n");

  sprintf(post, "\r\n--%s--\r\n", boundary);

  int filesize = get_filesize(filename);
  int length = filesize + strlen(pre) + strlen(post);

  sprintf(header, "POST /test/upload.php HTTP/1.0\r\n");
  sprintf(header + strlen(header), "Host: %s\r\n", hostname);
  sprintf(header + strlen(header), "Content-Type:multipart/form-data; ");
  sprintf(header + strlen(header), "boundary=%s\r\n", boundary);
  sprintf(header + strlen(header), "Content-Length: %d\r\n\r\n", length);

  write(sock, header, strlen(header));
  write(sock, pre, strlen(pre));
  send_file(sock, filename);
  write(sock, post, strlen(post));

  while(1){
    char buf[1024];

    int size = read(sock, buf, sizeof(buf));
    if(size <= 0){
      break;
    }

    buf[size] = 0;
    printf(">> %s\n", buf);
  }

  close(sock);
}

int main(int argc, char **argv)
{
  if(argc != 2){
    printf("usage: %s [filename]\n", argv[0]);
    exit(-1);
  }else{
    post_file("sdlabo.org", argv[1]);
  }
}
